import { ZodType, ZodError, z, ZodObject } from 'zod';

type ErrorMessage<T extends string> = T;
type Simplify<T> = {
    [P in keyof T]: T[P];
} & {};
type Impossible<T extends Record<string, any>> = Partial<Record<keyof T, never>>;
interface BaseOptions<TShared extends Record<string, ZodType>> {
    isServer?: boolean;
    shared?: TShared;
    onValidationError?: (error: ZodError) => never;
    onInvalidAccess?: (variable: string) => never;
    skipValidation?: boolean;
    emptyStringAsUndefined?: boolean;
}
interface LooseOptions<TShared extends Record<string, ZodType>> extends BaseOptions<TShared> {
    runtimeEnvStrict?: never;
    runtimeEnv: Record<string, string | boolean | number | undefined>;
}
interface StrictOptions<TPrefix extends string | undefined, TServer extends Record<string, ZodType>, TClient extends Record<string, ZodType>, TShared extends Record<string, ZodType>> extends BaseOptions<TShared> {
    runtimeEnvStrict: Record<{
        [TKey in keyof TClient]: TPrefix extends undefined ? never : TKey extends `${TPrefix}${string}` ? TKey : never;
    }[keyof TClient] | {
        [TKey in keyof TServer]: TPrefix extends undefined ? TKey : TKey extends `${TPrefix}${string}` ? never : TKey;
    }[keyof TServer] | {
        [TKey in keyof TShared]: TKey extends string ? TKey : never;
    }[keyof TShared], string | boolean | number | undefined>;
    runtimeEnv?: never;
}
interface ClientOptions<TPrefix extends string | undefined, TClient extends Record<string, ZodType>> {
    clientPrefix: TPrefix;
    client: Partial<{
        [TKey in keyof TClient]: TKey extends `${TPrefix}${string}` ? TClient[TKey] : ErrorMessage<`${TKey extends string ? TKey : never} is not prefixed with ${TPrefix}.`>;
    }>;
}
interface ServerOptions<TPrefix extends string | undefined, TServer extends Record<string, ZodType>> {
    server: Partial<{
        [TKey in keyof TServer]: TPrefix extends undefined ? TServer[TKey] : TPrefix extends "" ? TServer[TKey] : TKey extends `${TPrefix}${string}` ? ErrorMessage<`${TKey extends `${TPrefix}${string}` ? TKey : never} should not prefixed with ${TPrefix}.`> : TServer[TKey];
    }>;
}
type ServerClientOptions<TPrefix extends string | undefined, TServer extends Record<string, ZodType>, TClient extends Record<string, ZodType>> = (ClientOptions<TPrefix, TClient> & ServerOptions<TPrefix, TServer>) | (ServerOptions<TPrefix, TServer> & Impossible<ClientOptions<never, never>>) | (ClientOptions<TPrefix, TClient> & Impossible<ServerOptions<never, never>>);
type EnvOptions<TPrefix extends string | undefined, TServer extends Record<string, ZodType>, TClient extends Record<string, ZodType>, TShared extends Record<string, ZodType>> = (LooseOptions<TShared> & ServerClientOptions<TPrefix, TServer, TClient>) | (StrictOptions<TPrefix, TServer, TClient, TShared> & ServerClientOptions<TPrefix, TServer, TClient>);
declare function createEnv<TPrefix extends string | undefined, TServer extends Record<string, ZodType> = NonNullable<unknown>, TClient extends Record<string, ZodType> = NonNullable<unknown>, TShared extends Record<string, ZodType> = NonNullable<unknown>>(opts: EnvOptions<TPrefix, TServer, TClient, TShared>): Readonly<Simplify<z.infer<ZodObject<TServer>> & z.infer<ZodObject<TClient>> & z.infer<ZodObject<TShared>>>>;

export { type BaseOptions, type ClientOptions, type EnvOptions, type ErrorMessage, type LooseOptions, type ServerClientOptions, type ServerOptions, type Simplify, type StrictOptions, createEnv };
